package com.sd.business.service.bom.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.ReUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.tenant.entity.dict.po.DictCommonData;
import com.fjhx.tenant.service.dict.DictCommonDataService;
import com.ruoyi.common.constant.StatusConstant;
import com.ruoyi.common.core.domain.BaseIdPo;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.sd.business.entity.bom.constant.BomClassifyConstant;
import com.sd.business.entity.bom.dto.*;
import com.sd.business.entity.bom.enums.BomSpecPriceConfigCodeEnum;
import com.sd.business.entity.bom.po.Bom;
import com.sd.business.entity.bom.po.BomSpec;
import com.sd.business.entity.bom.po.BomSpecPriceConfig;
import com.sd.business.entity.bom.vo.BomSpecBoardSelectVo;
import com.sd.business.entity.bom.vo.BomSpecVo;
import com.sd.business.entity.department.constant.DepartmentConstant;
import com.sd.business.entity.inventory.po.Inventory;
import com.sd.business.entity.warehouse.constant.WarehouseConstant;
import com.sd.business.mapper.bom.BomSpecMapper;
import com.sd.business.service.bom.BomService;
import com.sd.business.service.bom.BomSpecPriceConfigService;
import com.sd.business.service.bom.BomSpecService;
import com.sd.business.service.inventory.InventoryService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.stream.Collectors;


/**
 * <p>
 * bom规格 服务实现类
 * </p>
 *
 * @author
 * @since 2023-06-30
 */
@Service
public class BomSpecServiceImpl extends ServiceImpl<BomSpecMapper, BomSpec> implements BomSpecService {
    @Autowired
    private InventoryService inventoryService;

    @Autowired
    private BomService bomService;

    @Autowired
    private DictCommonDataService dictCommonDataService;

    @Autowired
    private BomSpecPriceConfigService bomSpecPriceConfigService;

    @Override
    public Page<BomSpecVo> getPage(BomSpecSelectDto dto) {
        IWrapper<BomSpec> wrapper = getWrapper();
        wrapper.like("bs", BomSpec::getName, dto.getName());
        wrapper.like("bs", BomSpec::getCode, dto.getCode());
        wrapper.orderByDesc("bs", BomSpec::getId);
        Page<BomSpecVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        return page;
    }

    @Override
    public BomSpecVo detail(Long id) {
        BomSpec bomSpec = this.getById(id);
        BomSpecVo result = BeanUtil.toBean(bomSpec, BomSpecVo.class);
        return result;
    }

    @Override
    public void add(BomSpecDto bomSpecDto) {
        this.save(bomSpecDto);
    }

    @Override
    public void edit(BomSpecDto bomSpecDto) {
        this.updateById(bomSpecDto);
    }

    @Override
    public void delete(Long id) {
        this.removeById(id);
    }

    @Override
    public void editBoard(BomSpecUpdateBoardDto dto) {
        List<Long> bomSpecIdList = dto.getBomSpecIdList();
        this.update(q -> q.set(BomSpec::getIsShowBoard, StatusConstant.NO).set(BomSpec::getSort, null).eq(BomSpec::getIsShowBoard, StatusConstant.YES));
        if (ObjectUtil.isNotEmpty(bomSpecIdList)) {
            for (int i = 0; i < bomSpecIdList.size(); i++) {
                Long bomSpecId = bomSpecIdList.get(i);
                int sort = i;
                this.update(q -> q.set(BomSpec::getIsShowBoard, StatusConstant.YES)
                        .set(BomSpec::getSort, sort).eq(BomSpec::getId, bomSpecId));
            }
        }
    }

    @Override
    public List<BomSpecBoardSelectVo> getBoardList() {
        List<BomSpec> list = this.list(q -> q.eq(BomSpec::getIsShowBoard, StatusConstant.YES).orderByAsc(BomSpec::getSort));
        if (list.isEmpty()) {
            return Collections.emptyList();
        }
        List<Long> bomSpecIds = list.stream().map(BaseIdPo::getId).collect(Collectors.toList());
        // 查询半成品和包材库存
        Map<Long, Inventory> inventoryMap = inventoryService.mapKEntity(Inventory::getBomSpecId, q -> q.eq(Inventory::getDepartmentId, DepartmentConstant.SD_SPORTS)
                .in(Inventory::getWarehouseId, WarehouseConstant.SEMI_FINISHED_PRODUCT, WarehouseConstant.PACKAGING_MATERIAL)
                .in(Inventory::getBomSpecId, bomSpecIds));
        List<BomSpecBoardSelectVo> boardSelectList = BeanUtil.copyToList(list, BomSpecBoardSelectVo.class);
        List<BomSpecBoardSelectVo> collect = boardSelectList.stream().peek(item -> {
            Inventory inventory = inventoryMap.get(item.getId());
            if (inventory == null) {
                item.setInventoryQuantity(BigDecimal.ZERO);
            } else {
                BigDecimal inventoryQuantity = inventory.getQuantity().add(inventory.getLockQuantity() == null ? BigDecimal.ZERO : inventory.getLockQuantity());
                item.setInventoryQuantity(inventoryQuantity);
            }
        }).collect(Collectors.toList());
        return collect;
    }

    @Override
    public void editBoardSort(BomSpecUpdateBoardSortDto dto) {
        BomSpec bomSpec = this.getById(dto.getId());
        if (bomSpec != null && !Objects.equals(dto.getCurrentSort(), bomSpec.getSort())) {
            Integer oldSort = bomSpec.getSort();
            Integer currentSort = dto.getCurrentSort();
            // 如果当前排序大于原排序，当前排序到原排序之间 -1
            if (oldSort < currentSort) {
                this.update(q -> q.setSql("sort = sort - 1").le(BomSpec::getSort, currentSort).gt(BomSpec::getSort, oldSort));
            } else {
                this.update(q -> q.setSql("sort = sort + 1").lt(BomSpec::getSort, oldSort).ge(BomSpec::getSort, currentSort));
            }
            // 更新当前bom排序
            this.update(q -> q.set(BomSpec::getSort, currentSort).eq(BomSpec::getId, bomSpec.getId()));
        }
    }

    @Override
    public void editPrice(BomSpecDto bomSpecDto) {
        BomSpec bomSpec = this.getById(bomSpecDto.getId());
        if (bomSpec == null) {
            throw new ServiceException("未知bom规格id!");
        }
        bomSpec.setCostPrice(bomSpecDto.getCostPrice());
        bomSpec.setInternalSellingPrice(bomSpecDto.getInternalSellingPrice());
        bomSpec.setExternalSellingPrice(bomSpecDto.getExternalSellingPrice());
        this.updateById(bomSpec);
    }

    @Override
    public BigDecimal getPriceByParam(BomSpecPriceComputeDto dto) {
        if (dto.getLength() == null || dto.getWidth() == null  || dto.getHeight() == null) {
            return BigDecimal.ZERO;
        }
        Long bomClassifyId = dto.getBomClassifyId();
        if (!Objects.equals(bomClassifyId, BomClassifyConstant.YOGA_MAT) &&
                !Objects.equals(bomClassifyId, BomClassifyConstant.SKIP_MAT)) {
            return BigDecimal.ZERO;
        }
        // 是否是瑜伽垫
        boolean yogaMatFlag = Objects.equals(bomClassifyId, BomClassifyConstant.YOGA_MAT);
        // 厚度单位为cm，需要mm
        dto.setHeight(dto.getHeight().multiply(BigDecimal.TEN));
        // 获取bom字典值
        List<DictCommonData> dictCommonDataList = dictCommonDataService.list();
        Map<String, Map<String, String>> dictMap = dictCommonDataList.stream()
                .collect(Collectors.groupingBy(
                        DictCommonData::getDictCode,
                        Collectors.toMap(
                                DictCommonData::getDictKey,
                                DictCommonData::getDictValue)
                ));
        String level = dictMap.getOrDefault("bom_level", Collections.emptyMap()).get(dto.getLevel());
        String chromatophore = dictMap.getOrDefault("bom_chromatophore", Collections.emptyMap()).get(dto.getChromatophore());
        String embossingProcess = dictMap.getOrDefault("bom_embossingProcess", Collections.emptyMap()).get(dto.getEmbossingProcess());
        if (StrUtil.isBlank(level) || StrUtil.isBlank(chromatophore) || StrUtil.isBlank(embossingProcess)) {
            return BigDecimal.ZERO;
        }

        // 计算公式=（基材价格+品类+特性+颜色价格）*[（长+裁切配置）*（宽+裁切配置）/10000]*（厚度+压纹配置）*（1+损耗）+加工费+运费
        List<BomSpecPriceConfig> list = bomSpecPriceConfigService.list();
        Map<String, List<BomSpecPriceConfig>> configCodeMap = list.stream().collect(Collectors.groupingBy(BomSpecPriceConfig::getCode));

        // 计算bom每平方价格
        BigDecimal price = BigDecimal.ZERO;
        // 等级
        // 获取售价体系，10为体系一，20为体系二
        int bomType;
        if (Objects.equals(dto.getSellingPriceSystem(), "10")) {
            bomType = 0;
        } else {
            bomType = 1;
        }
        List<BomSpecPriceConfig> bomSpecMaterialConfigList = configCodeMap
                .getOrDefault(BomSpecPriceConfigCodeEnum.BASE_MATERIAL_PRICE.getKey(), Collections.emptyList());
        BomSpecPriceConfig levelDetail = bomSpecMaterialConfigList.stream()
                .filter(item -> item.getType() == bomType && item.getName().contains(level))
                .findAny().orElse(null);
        if (levelDetail == null) {
            return BigDecimal.ZERO;
        }
        price = price.add(levelDetail.getPrice());

        // 品类
        List<BomSpecPriceConfig> bomSpecCategoryConfigList = configCodeMap
                .getOrDefault(BomSpecPriceConfigCodeEnum.CATEGORY_PRICE.getKey(), Collections.emptyList());
        BomSpecPriceConfig categoryDetail = bomSpecCategoryConfigList.stream()
                .filter(item -> item.getName().contains(chromatophore))
                .findAny().orElse(null);
        if (categoryDetail == null) {
            return BigDecimal.ZERO;
        }
        price = price.add(categoryDetail.getPrice());

        // 颜色
        List<String> colourList = Arrays.asList(dto.getColour().split("\\+"));
        List<BomSpecPriceConfig> bomSpecColourConfigList = configCodeMap
                .getOrDefault(BomSpecPriceConfigCodeEnum.COLOUR_PRICE.getKey(), Collections.emptyList());

        for (String colour : colourList) {
            BigDecimal colourPrice = bomSpecColourConfigList.stream()
                    .filter(item -> item.getName().contains(colour))
                    .findAny()
                    .map(BomSpecPriceConfig::getPrice)
                    .orElse(BigDecimal.ZERO);
            if (colourList.size() > 1) {
                colourPrice = colourPrice.divide(BigDecimal.valueOf(2), 4, RoundingMode.HALF_UP);
            }
            price = price.add(colourPrice);
        }

        // 获取裁切配置
        List<BomSpecPriceConfig> bomSpecCropConfigList = configCodeMap
                .getOrDefault(BomSpecPriceConfigCodeEnum.CROP_CONFIG.getKey(), Collections.emptyList());
        BomSpecPriceConfig cropConfig = bomSpecCropConfigList.stream()
                .filter(item -> item.getName().contains(categoryDetail.getName()))
                .findAny().orElse(null);
        if (cropConfig != null) {
            dto.setLength(dto.getLength().add(cropConfig.getLength()));
            dto.setWidth(dto.getWidth().add(cropConfig.getWidth()));
        }

        // 获取压纹配置
        List<BomSpecPriceConfig> bomSpecEmbossingConfigList = configCodeMap.
                getOrDefault(BomSpecPriceConfigCodeEnum.EMBOSSING_CONFIG.getKey(), Collections.emptyList());
        BomSpecPriceConfig embossingConfig = bomSpecEmbossingConfigList.stream()
                .filter(item -> item.getName().contains(embossingProcess))
                .findAny().orElse(null);
        if (embossingConfig != null) {
            dto.setHeight(dto.getHeight().add(embossingConfig.getHeight()));
        }

        // 获取损耗配置
        BigDecimal depletion = BigDecimal.ZERO;
        String depletionCode;
        if (yogaMatFlag) {
            depletionCode = BomSpecPriceConfigCodeEnum.YOGA_MAT_DEPLETION_CONFIG.getKey();
        } else {
            depletionCode = BomSpecPriceConfigCodeEnum.SKIP_MAT_DEPLETION_CONFIG.getKey();
        }
        List<BomSpecPriceConfig> bomSpecDepletionConfigList = configCodeMap.getOrDefault(depletionCode, Collections.emptyList());
        BomSpecPriceConfig depletionConfig = bomSpecDepletionConfigList.stream()
                .filter(item -> item.getName().contains(categoryDetail.getName()))
                .findAny().orElse(null);
        if (depletionConfig != null) {
            depletion = depletionConfig.getDepletion();
        }
        // 获取附加配置
        List<BomSpecPriceConfig> additionalConfigList = bomSpecDepletionConfigList.stream()
                .filter(item -> Objects.equals(item.getType(), StatusConstant.YES)).collect(Collectors.toList());

        // 瑜伽垫对比宽度，跳绳垫对比级别
        BomSpecPriceConfig additionalConfig;
        if (yogaMatFlag) {
            additionalConfig = additionalConfigList.stream()
                    .filter(item -> this.operatorCompareTo(item.getName(), dto.getWidth()))
                    .findAny().orElse(null);
        } else {
            additionalConfig = additionalConfigList.stream()
                    .filter(item -> item.getName().contains(levelDetail.getName()))
                    .findAny().orElse(null);
        }

        // 附加配置匹配成功，增加损耗
        if (additionalConfig != null) {
            depletion = depletion.add(additionalConfig.getDepletion());
        }

        // 获取加工费配置
        List<BomSpecPriceConfig> bomSpecProcessingFeeConfigList = configCodeMap
                .getOrDefault(BomSpecPriceConfigCodeEnum.PROCESSING_FEE_CONFIG.getKey(), Collections.emptyList());
        BigDecimal processingFee = bomSpecProcessingFeeConfigList.stream()
                .map(BomSpecPriceConfig::getPrice)
                .reduce(BigDecimal.ZERO, BigDecimal::add);
        // 如果是热复合时，增加复合费用
        if (Objects.equals(categoryDetail.getName(), "双层热复合")) {
            List<BomSpecPriceConfig> compoundingCostConfigList = configCodeMap
                    .getOrDefault(BomSpecPriceConfigCodeEnum.COMPOUNDING_COST_CONFIG.getKey(), Collections.emptyList());
            BomSpecPriceConfig priceConfig = compoundingCostConfigList.stream()
                    .filter(item -> this.operatorCompareTo(item.getName(), dto.getWidth()))
                    .findAny().orElse(null);
            if (priceConfig != null) {
                processingFee = processingFee.add(priceConfig.getPrice());
            }
        }

        // 获取运费配置
        List<BomSpecPriceConfig> bomSpecFreightConfigList = configCodeMap
                .getOrDefault(BomSpecPriceConfigCodeEnum.FREIGHT_CONFIG.getKey(), Collections.emptyList());
        BigDecimal freightFee = BigDecimal.ZERO;
        BomSpecPriceConfig freightConfig = bomSpecFreightConfigList.stream()
                .filter(item -> this.operatorCompareTo(item.getName(), dto.getWidth())).findAny().orElse(null);
        if (freightConfig != null) {
            freightFee = freightConfig.getPrice();
        }

        return price.multiply(dto.getLength().multiply(dto.getWidth()).divide(BigDecimal.valueOf(10000), 10, RoundingMode.HALF_UP))
                .multiply(dto.getHeight())
                .multiply(BigDecimal.ONE.add(depletion.multiply(BigDecimal.valueOf(0.01))))
                .add(processingFee)
                .add(freightFee)
                .setScale(2, RoundingMode.HALF_UP);
    }

    @Override
    public BigDecimal getPriceById(Long id) {
        BomSpec bomSpec = this.getById(id);
        if (bomSpec == null) {
            throw new ServiceException("未知bom规格id!");
        }
        Bom bom = bomService.getById(bomSpec.getBomId());
        // 添加计算条件
        BomSpecPriceComputeDto dto = new BomSpecPriceComputeDto();
        dto.setBomClassifyId(bom.getBomClassifyId());
        dto.setChromatophore(bom.getChromatophore());
        dto.setSellingPriceSystem(bom.getSellingPriceSystem());
        dto.setLevel(bom.getLevel());
        dto.setEmbossingProcess(bom.getEmbossingProcess());
        dto.setColour(bomSpec.getColour());
        dto.setLength(bomSpec.getLength());
        dto.setWidth(bomSpec.getWidth());
        dto.setHeight(bomSpec.getHeight());

        BigDecimal price = this.getPriceByParam(dto);
        // 计算不出价格时，取当前成本价
        if (Objects.equals(price, BigDecimal.ZERO)) {
            price = bomSpec.getCostPrice();
        }
        return price;
    }

    /**
     * 对比
     * @param rule 规则
     * @param value
     * @return
     */
    private boolean operatorCompareTo(String rule, BigDecimal value) {
        Integer number = ReUtil.getFirstNumber(rule);
        if (rule.startsWith(">=") || rule.startsWith("≥")) {
            return value.compareTo(BigDecimal.valueOf(number)) >= 0;
        } else if (rule.startsWith(">")) {
            return value.compareTo(BigDecimal.valueOf(number)) > 0;
        } else if (rule.startsWith("<=") || rule.startsWith("≤")) {
            return value.compareTo(BigDecimal.valueOf(number)) <= 0;
        } else if (rule.startsWith("<")) {
            return value.compareTo(BigDecimal.valueOf(number)) < 0;
        } else {
            return value.compareTo(BigDecimal.valueOf(number)) == 0;
        }
    }
}
