package com.sd.business.service.order.impl;

import cn.hutool.core.util.StrUtil;
import com.ruoyi.common.utils.SecurityUtils;
import com.sd.business.entity.department.po.Department;
import com.sd.business.entity.order.po.OrderInfo;
import com.sd.business.entity.order.po.OrderProofingRegistration;
import com.sd.business.entity.order.po.OrderSku;
import com.sd.business.mapper.order.OrderProofingRegistrationMapper;
import com.sd.business.service.order.OrderProofingRegistrationService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.sd.business.service.order.OrderService;
import com.sd.business.service.order.OrderSkuService;
import com.sd.framework.util.Assert;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.sd.business.entity.order.vo.OrderProofingRegistrationVo;
import com.sd.business.entity.order.dto.OrderProofingRegistrationSelectDto;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.sd.business.entity.order.dto.OrderProofingRegistrationDto;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;

/**
 * <p>
 * 订单打样登记 服务实现类
 * </p>
 *
 * @author 
 * @since 2023-12-19
 */
@Service
public class OrderProofingRegistrationServiceImpl extends ServiceImpl<OrderProofingRegistrationMapper, OrderProofingRegistration> implements OrderProofingRegistrationService {

    @Autowired
    private OrderService orderService;

    @Autowired
    private OrderSkuService orderSkuService;

    @Override
    public Page<OrderProofingRegistrationVo> getPage(OrderProofingRegistrationSelectDto dto) {
        IWrapper<OrderProofingRegistration> wrapper = getWrapper();
        wrapper.orderByDesc("opr", OrderProofingRegistration::getId);
        wrapper.eq("d", Department::getId, dto.getDepartmentId());
        wrapper.and(StrUtil.isNotBlank(dto.getOrderCode()), q -> q
                .like("oi", OrderInfo::getCode, dto.getOrderCode())
                .or()
                .like("oi", OrderInfo::getWlnCode, dto.getOrderCode()));
        Page<OrderProofingRegistrationVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        return page;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void add(OrderProofingRegistrationDto dto) {
        OrderInfo orderInfo = orderService.getById(dto.getOrderId());
        Assert.notNull(orderInfo, "未知订单");

        dto.setOperator(SecurityUtils.getLoginUser().getUser().getNickName());
        this.save(dto);

        // 打样金额
        BigDecimal proofingFee = dto.getUnitPrice().add(dto.getCustomProcessingFee());
        // 取出订单任意一个sku，将打样金额保存
        OrderSku orderSku = orderSkuService.getOne(q -> q.eq(OrderSku::getOrderId, orderInfo.getId()));
        orderSku.setProofingFee(orderSku.getProofingFee().add(proofingFee));
        orderInfo.setProofingFee(orderInfo.getProofingFee().add(proofingFee));
        orderInfo.setTotalAmount(orderInfo.getTotalAmount().add(proofingFee));
        orderService.updateById(orderInfo);
        orderSkuService.updateById(orderSku);
    }

    @Override
    public void edit(OrderProofingRegistrationDto dto) {
        this.updateById(dto);
    }

    @Override
    public void delete(Long id) {
        this.removeById(id);
    }

}
